"""HTTP client wrapper for Vaulty API."""

from typing import Any

import httpx

from .exceptions import (
    VaultyAPIError,
    VaultyAuthenticationError,
    VaultyAuthorizationError,
    VaultyNotFoundError,
    VaultyRateLimitError,
    VaultyValidationError,
)
from .logging import get_logger, sanitize_sensitive_data

logger = get_logger(__name__)


class HTTPClient:
    """HTTP client wrapper for Vaulty API."""

    def __init__(
        self,
        base_url: str,
        api_token: str | None = None,
        jwt_token: str | None = None,
        timeout: float = 30.0,
        api_version: str = "v1",
    ):
        self.base_url = base_url.rstrip("/")
        self.api_token = api_token
        self.jwt_token = jwt_token
        self.timeout = timeout
        self.api_version = api_version

        # Determine auth header
        if api_token:
            self.auth_header = f"Bearer {api_token}"
        elif jwt_token:
            self.auth_header = f"Bearer {jwt_token}"
        else:
            self.auth_header = None

        self._client: httpx.AsyncClient | None = None

    async def _get_client(self) -> httpx.AsyncClient:
        """Get or create HTTP client."""
        if self._client is None:
            headers = {
                "Content-Type": "application/json",
                "API-Version": self.api_version,
            }
            if self.auth_header:
                headers["Authorization"] = self.auth_header

            self._client = httpx.AsyncClient(
                base_url=self.base_url, headers=headers, timeout=self.timeout
            )
        return self._client

    async def close(self):
        """Close HTTP client."""
        if self._client:
            await self._client.aclose()
            self._client = None

    def _raise_for_status(self, response: httpx.Response):
        """Raise appropriate exception for error status codes."""
        if response.is_success:
            return

        status_code = response.status_code
        detail = None

        try:
            error_data = response.json()
            detail = error_data.get("detail", str(error_data))
        except Exception:
            detail = response.text or f"HTTP {status_code}"

        # Log error before raising
        logger.warning(
            f"HTTP {status_code} error: {detail}",
            extra={"status_code": status_code, "detail": detail},
        )

        if status_code == 401:
            raise VaultyAuthenticationError(f"Authentication failed: {detail}", status_code, detail)
        if status_code == 403:
            raise VaultyAuthorizationError(
                f"Insufficient permissions: {detail}", status_code, detail
            )
        if status_code == 404:
            raise VaultyNotFoundError(f"Resource not found: {detail}", status_code, detail)
        if status_code == 400:
            raise VaultyValidationError(f"Validation error: {detail}", status_code, detail)
        if status_code == 429:
            retry_after = None
            if "Retry-After" in response.headers:
                try:
                    retry_after = int(response.headers["Retry-After"])
                except ValueError:
                    pass
            raise VaultyRateLimitError(
                f"Rate limit exceeded: {detail}", status_code, detail, retry_after
            )
        raise VaultyAPIError(f"API error: {detail}", status_code, detail)

    async def request(
        self,
        method: str,
        path: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs,
    ) -> httpx.Response:
        """Make HTTP request."""
        client = await self._get_client()

        # Ensure path starts with /
        if not path.startswith("/"):
            path = "/" + path

        # Log request (sanitize sensitive data)
        logger.debug(
            f"Making {method} request to {path}",
            extra={
                "method": method,
                "path": path,
                "params": params,
                "json": sanitize_sensitive_data(json) if json else None,
            },
        )

        try:
            response = await client.request(
                method=method, url=path, params=params, json=json, **kwargs
            )

            logger.debug(
                f"Response {response.status_code} for {method} {path}",
                extra={"status_code": response.status_code, "path": path},
            )

            self._raise_for_status(response)
            return response
        except Exception as e:
            logger.error(
                f"Request failed: {method} {path}",
                exc_info=True,
                extra={"method": method, "path": path, "error": str(e)},
            )
            raise

    async def get(
        self, path: str, params: dict[str, Any] | None = None, **kwargs
    ) -> httpx.Response:
        """GET request."""
        return await self.request("GET", path, params=params, **kwargs)

    async def post(self, path: str, json: dict[str, Any] | None = None, **kwargs) -> httpx.Response:
        """POST request."""
        return await self.request("POST", path, json=json, **kwargs)

    async def put(self, path: str, json: dict[str, Any] | None = None, **kwargs) -> httpx.Response:
        """PUT request."""
        return await self.request("PUT", path, json=json, **kwargs)

    async def patch(
        self, path: str, json: dict[str, Any] | None = None, **kwargs
    ) -> httpx.Response:
        """PATCH request."""
        return await self.request("PATCH", path, json=json, **kwargs)

    async def delete(self, path: str, **kwargs) -> httpx.Response:
        """DELETE request."""
        return await self.request("DELETE", path, **kwargs)



"""CLI utilities."""

import asyncio
import os
import sys

import click

from .. import VaultyClient
from .config import CLIConfig


def get_client(
    token: str | None = None, base_url: str | None = None, non_interactive: bool = True
) -> VaultyClient:
    """Get Vaulty client from config or parameters.

    Args:
        token: API token (overrides stored credentials)
        base_url: Base URL (overrides stored/configured URL)
        non_interactive: Non-interactive mode (default: True)

    Returns:
        VaultyClient instance
    """
    config = CLIConfig()
    config_data = config.load()

    # Determine base URL (priority: parameter > env var > stored > default)
    if base_url:
        api_base_url = base_url.rstrip("/")
    elif os.getenv("VAULTY_API_URL"):
        api_base_url = os.getenv("VAULTY_API_URL").rstrip("/")
    elif config_data.get("api_url"):
        api_base_url = config_data["api_url"].rstrip("/")
    else:
        api_base_url = "https://api.vaulty.com"

    # Normalize base URL - remove trailing /api if present (paths include /api/v1/...)
    # Backend structure: Health at root (/health), API under /api/v1/...
    # So base_url should be at root (http://localhost:8000), not http://localhost:8000/api
    if api_base_url.endswith("/api"):
        api_base_url = api_base_url[:-4]
    elif api_base_url.endswith("/api/"):
        api_base_url = api_base_url[:-5]

    # Determine token (priority: parameter > env var > stored)
    api_token = None
    jwt_token = None

    if token:
        api_token = token
    elif os.getenv("VAULTY_API_TOKEN"):
        api_token = os.getenv("VAULTY_API_TOKEN")
    elif config_data.get("api_token"):
        api_token = config_data["api_token"]
    elif os.getenv("VAULTY_JWT_TOKEN"):
        jwt_token = os.getenv("VAULTY_JWT_TOKEN")
    elif config_data.get("jwt_token"):
        jwt_token = config_data["jwt_token"]

    if not api_token and not jwt_token:
        if non_interactive:
            raise ValueError(
                "No authentication token found. Set VAULTY_API_TOKEN or run 'vaulty login'"
            )
        raise ValueError("No authentication token found. Run 'vaulty login' first.")

    return VaultyClient(base_url=api_base_url, api_token=api_token, jwt_token=jwt_token)


async def get_project_from_token_scope(client: VaultyClient) -> dict[str, str] | None:
    """Extract project ID and name from token scope (for project-scoped tokens).

    For project-scoped tokens, extracts project ID from token scope and attempts
    to fetch the project name. The server's secrets route can fetch projects by ID,
    so we use that to get project information.

    For full-scope tokens, returns None (project must be specified explicitly).

    Args:
        client: VaultyClient instance

    Returns:
        Dict with 'id' and 'name' keys if token is project-scoped, None otherwise.
        'name' may be None if we can't fetch it, but 'id' will always be present.
    """
    try:
        # Try to get project from token scope by listing tokens
        tokens_result = await client.tokens.list(page=1, page_size=1)

        if tokens_result.items:
            token = tokens_result.items[0]
            scope = token.scope

            # Extract project ID from scope (format: project:p-xxxxx:read/write)
            if scope.startswith("project:"):
                parts = scope.split(":")
                if len(parts) >= 2:
                    project_id = parts[1]

                    # For project-scoped tokens, we have the project ID from the token scope
                    # The server's secrets route accepts project_name parameter, and if it matches
                    # the token's project_id, it will fetch the project by ID internally
                    # So we can use the project ID as the project name in API calls
                    # This maintains the standard approach of using project_name parameter
                    return {
                        "id": project_id,
                        "name": project_id,  # Use ID as name - server's secrets route accepts it
                    }

        return None
    except Exception:
        # If tokens.list fails, return None
        return None


def run_async(coro):
    """Run async coroutine in sync context."""
    try:
        loop = asyncio.get_event_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

    return loop.run_until_complete(coro)


def detect_cicd() -> bool:
    """Detect if running in CI/CD environment."""
    ci_vars = [
        "CI",
        "CONTINUOUS_INTEGRATION",
        "GITHUB_ACTIONS",
        "GITLAB_CI",
        "JENKINS_URL",
        "CIRCLECI",
    ]
    return any(os.getenv(var) for var in ci_vars)


def resolve_project(
    project: str | None, client: VaultyClient, required: bool = False
) -> str | None:
    """Resolve project name from various sources.

    Args:
        project: Explicitly provided project name
        client: VaultyClient instance
        required: Whether project is required (for full-scope tokens)

    Returns:
        Project name or None if not found and not required

    Raises:
        ValueError: If project is required but cannot be determined
    """
    if project:
        return project

    # Try environment variable
    project = os.getenv("VAULTY_PROJECT")
    if project:
        return project

    # Try to infer from token scope (for project-scoped tokens)
    project_info = run_async(get_project_from_token_scope(client))
    if project_info:
        return project_info.get("name") or project_info.get("id")

    if required:
        raise ValueError(
            "--project is required for full-scope tokens. "
            "For project-scoped tokens, project is auto-detected."
        )

    return None


def handle_cli_errors(func):
    """Decorator to handle common CLI errors.

    Usage:
        @handle_cli_errors
        def my_command(...):
            ...
    """
    import functools

    from .. import (
        VaultyAPIError,
        VaultyAuthenticationError,
        VaultyAuthorizationError,
        VaultyNotFoundError,
        VaultyRateLimitError,
        VaultyValidationError,
    )

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except VaultyNotFoundError as e:
            click.echo(f"Error: Resource not found - {e.detail or str(e)}", err=True)
            sys.exit(1)
        except VaultyAuthenticationError:
            click.echo("Error: Authentication failed. Please run 'vaulty login'", err=True)
            sys.exit(1)
        except VaultyAuthorizationError as e:
            click.echo(f"Error: Insufficient permissions - {e.detail or str(e)}", err=True)
            sys.exit(1)
        except VaultyValidationError as e:
            click.echo(f"Error: Validation failed - {e.detail or str(e)}", err=True)
            sys.exit(1)
        except VaultyRateLimitError as e:
            click.echo(
                f"Error: Rate limit exceeded. {e.detail or 'Please try again later.'}", err=True
            )
            sys.exit(1)
        except VaultyAPIError as e:
            click.echo(f"Error: {e.detail or str(e)}", err=True)
            sys.exit(1)
        except ValueError as e:
            click.echo(f"Error: {e}", err=True)
            sys.exit(2)
        except Exception as e:
            click.echo(f"Unexpected error: {e}", err=True)
            sys.exit(1)

    return wrapper



"""Token resource client."""

from ..http import HTTPClient
from ..models import (
    PaginatedResponse,
    TokenResponse,
)
from ..retry import RetryConfig, retry_with_backoff


class TokenResource:
    """Client for token management operations."""

    def __init__(self, http_client: HTTPClient, retry_config: RetryConfig | None = None):
        self.http_client = http_client
        self.retry_config = retry_config

    async def create(
        self, scope: str, description: str | None = None, password: str | None = None
    ) -> TokenResponse:
        """Create API token (full scope or project-scoped).

        Args:
            scope: Token scope ("full", "read", "write", or "project:{project_id}:read/write")
            description: Optional token description
            password: Customer password (required for first token if no DEK exists)

        Returns:
            TokenResponse with token data (includes token value on creation)
        """

        async def _create():
            data = {"scope": scope}
            if description:
                data["description"] = description
            if password:
                data["password"] = password

            response = await self.http_client.post("/api/v1/tokens", json=data)
            return TokenResponse(**response.json())

        return await retry_with_backoff(_create, self.retry_config)

    async def list(self, page: int = 1, page_size: int = 50) -> PaginatedResponse[TokenResponse]:
        """List tokens with pagination.

        Args:
            page: Page number (1-indexed)
            page_size: Number of items per page (1-100)

        Returns:
            PaginatedResponse with tokens
        """

        async def _list():
            response = await self.http_client.get(
                "/api/v1/tokens", params={"page": page, "page_size": page_size}
            )
            data = response.json()

            # Handle both paginated response and direct list
            if isinstance(data, list):
                # Direct list response
                items = [TokenResponse(**item) for item in data]
                total = len(items)
                total_pages = 1 if total <= page_size else (total + page_size - 1) // page_size
                return PaginatedResponse[TokenResponse](
                    items=items,
                    total=total,
                    page=page,
                    page_size=page_size,
                    total_pages=total_pages,
                    has_next=page < total_pages,
                    has_previous=page > 1,
                )
            # Paginated response
            return PaginatedResponse[TokenResponse](
                items=[TokenResponse(**item) for item in data["items"]],
                total=data["total"],
                page=data["page"],
                page_size=data["page_size"],
                total_pages=data["total_pages"],
                has_next=data["has_next"],
                has_previous=data["has_previous"],
            )

        return await retry_with_backoff(_list, self.retry_config)

    async def delete(self, token_id: str) -> None:
        """Delete token.

        Args:
            token_id: Token ID
        """

        async def _delete():
            await self.http_client.delete(f"/api/v1/tokens/{token_id}")

        await retry_with_backoff(_delete, self.retry_config)



"""Main VaultyClient class."""

import os

from .auth import AuthHandler
from .http import HTTPClient
from .resources import (
    ActivityResource,
    CustomerResource,
    HealthResource,
    ProjectResource,
    SecretResource,
    TokenResource,
)
from .retry import RetryConfig


class VaultyClient:
    """Main client for Vaulty API.

    This is the primary interface for interacting with the Vaulty API. It provides
    access to all resources (customers, projects, secrets, tokens, activities, health)
    through convenient attributes.

    The client supports both API tokens (full-scope or project-scoped) and JWT tokens
    for authentication. It includes automatic retry logic with exponential backoff
    and rate limit handling.

    Example:
        >>> # Initialize with API token
        >>> client = VaultyClient(
        ...     base_url="https://api.vaulty.com",
        ...     api_token="vaulty_abc123..."
        ... )
        >>>
        >>> # Use async context manager for automatic cleanup
        >>> async with VaultyClient(api_token="vaulty_abc123...") as client:
        ...     secret = await client.secrets.get_value(
        ...         project_name="my-project",
        ...         key="API_KEY"
        ...     )
        ...     print(secret.value)
    """

    def __init__(
        self,
        base_url: str = "https://api.vaulty.com",
        api_token: str | None = None,
        jwt_token: str | None = None,
        email: str | None = None,
        password: str | None = None,
        timeout: float = 30.0,
        max_retries: int = 3,
        retry_backoff_factor: float = 2.0,
        rate_limit_retry: bool = True,
        api_version: str = "v1",
    ):
        """Initialize Vaulty client.

        Args:
            base_url: API base URL (default: https://api.vaulty.com)
            api_token: API token (full scope or project-scoped).
                       Tokens starting with 'vaulty_' are API tokens.
            jwt_token: JWT token obtained from login. Takes precedence over api_token.
            email: Customer email (for JWT login). Note: login() must be called explicitly.
            password: Customer password (for JWT login). Note: login() must be called explicitly.
            timeout: Request timeout in seconds (default: 30.0)
            max_retries: Maximum number of retry attempts for failed requests (default: 3)
            retry_backoff_factor: Exponential backoff multiplier (default: 2.0)
            rate_limit_retry: Enable automatic retry on rate limit errors (default: True)
            api_version: API version string (default: "v1")

        Note:
            If both api_token and jwt_token are provided, jwt_token takes precedence.
            For email/password login, call client.auth.login(email, password) explicitly.

        Raises:
            ValueError: If invalid configuration is provided
        """
        # Create HTTP client
        self.http_client = HTTPClient(
            base_url=base_url,
            api_token=api_token,
            jwt_token=jwt_token,
            timeout=timeout,
            api_version=api_version,
        )

        # Create auth handler
        self.auth = AuthHandler(self.http_client)

        # Create retry config
        self.retry_config = RetryConfig(
            max_retries=max_retries, backoff_factor=retry_backoff_factor
        )

        # Create resource clients
        self.customers = CustomerResource(self.http_client, self.retry_config)
        self.projects = ProjectResource(self.http_client, self.retry_config)
        self.secrets = SecretResource(self.http_client, self.retry_config)
        self.tokens = TokenResource(self.http_client, self.retry_config)
        self.activities = ActivityResource(self.http_client, self.retry_config)
        self.health = HealthResource(self.http_client, self.retry_config)

        # Handle email/password login
        if email and password:
            # This will be async, so we'll need to handle it differently
            # For now, users should call login() explicitly
            pass

    @classmethod
    def from_config(cls) -> "VaultyClient":
        """Load client from configuration file.

        Loads configuration from:
        1. Environment variables (highest priority)
        2. ~/.vaulty/config.yaml file
        3. Encrypted credentials file

        Returns:
            VaultyClient instance

        Raises:
            ValueError: If no authentication token is found
        """
        from pathlib import Path

        import yaml

        # Default values
        base_url = "https://api.vaulty.com"
        api_token = None
        jwt_token = None

        # Load from config file if it exists
        config_file = Path.home() / ".vaulty" / "config.yaml"
        if config_file.exists():
            try:
                with open(config_file) as f:
                    config_data = yaml.safe_load(f) or {}
                    if config_data.get("api_url"):
                        base_url = config_data["api_url"]
                    if config_data.get("api_token"):
                        api_token = config_data["api_token"]
                    if config_data.get("jwt_token"):
                        jwt_token = config_data["jwt_token"]
            except Exception:
                # If config file is invalid, continue with defaults
                pass

        # Override with environment variables (highest priority)
        if os.getenv("VAULTY_API_URL"):
            base_url = os.getenv("VAULTY_API_URL")
        if os.getenv("VAULTY_API_TOKEN"):
            api_token = os.getenv("VAULTY_API_TOKEN")
        if os.getenv("VAULTY_JWT_TOKEN"):
            jwt_token = os.getenv("VAULTY_JWT_TOKEN")

        # Try to load from encrypted credentials if no token found
        if not api_token and not jwt_token:
            from .cli.config import CLIConfig

            config = CLIConfig()
            auth_info = config.get_auth_info()
            if auth_info:
                if auth_info.get("type") == "api_token":
                    api_token = auth_info.get("token")
                elif auth_info.get("type") == "jwt":
                    jwt_token = auth_info.get("token")
                if auth_info.get("base_url"):
                    base_url = auth_info.get("base_url")

        if not api_token and not jwt_token:
            raise ValueError(
                "No authentication token found. "
                "Set VAULTY_API_TOKEN or VAULTY_JWT_TOKEN environment variable, "
                "or run 'vaulty login'"
            )

        return cls(base_url=base_url, api_token=api_token, jwt_token=jwt_token)

    @classmethod
    def from_env(cls) -> "VaultyClient":
        """Load client from environment variables.

        Returns:
            VaultyClient instance
        """
        base_url = os.getenv("VAULTY_API_URL", "https://api.vaulty.com")
        api_token = os.getenv("VAULTY_API_TOKEN")
        jwt_token = os.getenv("VAULTY_JWT_TOKEN")

        if api_token:
            return cls(base_url=base_url, api_token=api_token)
        if jwt_token:
            return cls(base_url=base_url, jwt_token=jwt_token)
        raise ValueError("VAULTY_API_TOKEN or VAULTY_JWT_TOKEN environment variable required")

    async def __aenter__(self):
        """Async context manager entry."""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit."""
        await self.close()

    async def close(self):
        """Close HTTP client connections.

        Closes all HTTP connections and cleans up resources. This should be called
        when the client is no longer needed, or use the async context manager.

        Example:
            >>> client = VaultyClient(api_token="vaulty_abc123...")
            >>> # ... use client ...
            >>> await client.close()
        """
        await self.http_client.close()



"""Authentication commands."""

import sys

import click

from ... import VaultyAPIError, VaultyClient
from ..config import CLIConfig
from ..utils import get_client, run_async


@click.group()
def auth_group():
    """Authentication commands."""


@auth_group.command()
@click.argument("token", required=False)
@click.option("--email", "-e", help="Email address (for JWT login)")
@click.option("--password", "-p", help="Password (prompted if not provided)")
@click.option("--token", "-t", "token_flag", help="API token (alternative to positional)")
@click.option("--token-env", help="Environment variable containing API token")
@click.option("--token-file", help="File containing API token")
@click.option("--project", help="Project name (for project-scoped tokens)")
@click.option("--base-url", "-u", help="Base URL for API (e.g., http://localhost:3001)")
def login(token, email, password, token_flag, token_env, token_file, project, base_url):
    """Login and store credentials and base URL.

    Default usage: vaulty login <token> [--base-url <url>]
    """
    import os

    config = CLIConfig()

    # Determine base URL (priority: --base-url > env var > default)
    if base_url:
        api_base_url = base_url
    elif os.getenv("VAULTY_API_URL"):
        api_base_url = os.getenv("VAULTY_API_URL")
    else:
        api_base_url = "https://api.vaulty.com"

    # Determine authentication method
    api_token = None

    if token:  # Positional argument (default)
        api_token = token
    elif token_flag:  # --token flag
        api_token = token_flag
    elif token_env:
        api_token = os.getenv(token_env)
        if not api_token:
            click.echo(f"Error: Environment variable {token_env} not set", err=True)
            sys.exit(1)
    elif token_file:
        try:
            with open(token_file) as f:
                api_token = f.read().strip()
        except FileNotFoundError:
            click.echo(f"Error: Token file not found: {token_file}", err=True)
            sys.exit(1)
    elif email:
        if not password:
            password = click.prompt("Password", hide_input=True)

        # Login with email/password to get JWT
        try:
            client = VaultyClient(base_url=api_base_url)
            token_response = run_async(client.customers.login(email, password))
            # Store JWT token and base URL
            config.save_jwt_token(
                token_response["access_token"], email=email, base_url=api_base_url
            )
            click.echo("Logged in successfully!")
            click.echo(f"Base URL: {api_base_url}")
            return
        except VaultyAPIError as e:
            click.echo(f"Error: {e.detail}", err=True)
            sys.exit(1)
    else:
        click.echo("Error: Must provide token, --email, --token-env, or --token-file", err=True)
        click.echo("Usage: vaulty login <token> [--base-url <url>]", err=True)
        sys.exit(2)

    # Store API token and base URL
    config.save_api_token(api_token, project=project, base_url=api_base_url)
    click.echo("Logged in successfully!")
    click.echo(f"Base URL: {api_base_url}")

    # Verify token and show info
    try:
        client = VaultyClient(api_token=api_token, base_url=api_base_url)
        customer = run_async(client.customers.get_current())
        click.echo(f"Authenticated as: {customer.email}")
        if project:
            click.echo(f"Project scope: {project}")
    except VaultyAPIError as e:
        click.echo(f"Warning: Could not verify token: {e.detail}", err=True)


@auth_group.command()
def logout():
    """Logout (clear stored credentials and base URL)."""
    config = CLIConfig()
    config.clear_credentials()
    click.echo("Logged out successfully!")


@auth_group.command()
def status():
    """Show current authentication status."""
    config = CLIConfig()
    auth_info = config.get_auth_info()

    if auth_info:
        click.echo(f"Type: {auth_info.get('type', 'unknown')}")
        if auth_info.get("base_url"):
            click.echo(f"Base URL: {auth_info['base_url']}")
        if auth_info.get("project"):
            click.echo(f"Project: {auth_info['project']}")
        click.echo("Status: Authenticated")
    else:
        click.echo("Status: Not authenticated")


@auth_group.command()
def info():
    """Show current token info (scope, project, base URL, etc.)."""
    try:
        client = get_client()
        customer = run_async(client.customers.get_current())
        click.echo(f"Email: {customer.email}")
        click.echo(f"Customer ID: {customer.id}")

        config = CLIConfig()
        auth_info = config.get_auth_info()
        if auth_info:
            if auth_info.get("base_url"):
                click.echo(f"Base URL: {auth_info['base_url']}")
            if auth_info.get("project"):
                click.echo(f"Project: {auth_info['project']}")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@auth_group.command()
def validate():
    """Validate token (CI/CD friendly, exit code 0 if valid, 1 if invalid)."""
    try:
        client = get_client()
        run_async(client.customers.get_current())
        sys.exit(0)
    except Exception:
        sys.exit(1)



"""Token commands."""

import sys

import click

from ...cli.output import OutputFormatter
from ...cli.utils import get_client, run_async


@click.group()
def tokens_group():
    """Token management commands."""


@tokens_group.command("list")
@click.option("--page", default=1, help="Page number")
@click.option("--page-size", default=50, help="Items per page")
@click.option(
    "--format", "-f", default="table", type=click.Choice(["json", "yaml", "plain", "table"])
)
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def list_tokens(page, page_size, format, token, base_url):
    """List all tokens."""
    try:
        client = get_client(token=token, base_url=base_url)
        result = run_async(client.tokens.list(page=page, page_size=page_size))

        formatter = OutputFormatter(format=format)
        output = formatter.format_output(
            {
                "items": [item.dict() for item in result.items],
                "total": result.total,
                "page": result.page,
                "page_size": result.page_size,
                "total_pages": result.total_pages,
                "has_next": result.has_next,
                "has_previous": result.has_previous,
            }
        )
        click.echo(output)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@tokens_group.command("create")
@click.argument("name")
@click.option(
    "--scope",
    required=True,
    help="Token scope (full, read, write, or project:{project_id}:read/write)",
)
@click.option("--description", help="Token description")
@click.option("--password", help="Customer password (required for first token if no DEK exists)")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def create_token(name, scope, description, password, format, token, base_url):
    """Create API token."""
    try:
        client = get_client(token=token, base_url=base_url)
        token_response = run_async(
            client.tokens.create(scope=scope, description=description or name, password=password)
        )

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(token_response.dict()))

        if token_response.token:
            click.echo(f"\nToken: {token_response.token}", err=True)
            click.echo("⚠️  Save this token now - it won't be shown again!", err=True)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@tokens_group.command("delete")
@click.argument("token_id")
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def delete_token(token_id, token, base_url):
    """Delete token."""
    try:
        client = get_client(token=token, base_url=base_url)
        run_async(client.tokens.delete(token_id))
        click.echo("Token deleted successfully!")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Pydantic models for SDK."""

from datetime import datetime
from typing import Any, Generic, TypeVar

from pydantic import BaseModel

T = TypeVar("T")


# Pagination Models
class PaginatedResponse(BaseModel, Generic[T]):
    """Paginated response wrapper."""

    items: list[T]
    total: int
    page: int
    page_size: int
    total_pages: int
    has_next: bool
    has_previous: bool


# Customer Models
class CustomerCreate(BaseModel):
    """Customer creation request."""

    email: str
    password: str


class CustomerResponse(BaseModel):
    """Customer response."""

    id: str
    email: str
    created_at: datetime | None = None
    updated_at: datetime | None = None


class CustomerSettingsUpdate(BaseModel):
    """Customer settings update request."""

    rate_limit_enabled: bool | None = None
    rate_limit_requests_per_minute: int | None = None


class CustomerSettingsResponse(BaseModel):
    """Customer settings response."""

    rate_limit_enabled: bool
    rate_limit_requests_per_minute: int


# Project Models
class ProjectCreate(BaseModel):
    """Project creation request."""

    name: str
    description: str | None = None


class ProjectUpdate(BaseModel):
    """Project update request."""

    description: str | None = None


class ProjectResponse(BaseModel):
    """Project response."""

    id: str | None = None
    name: str
    description: str | None = None
    created_at: datetime | None = None
    updated_at: datetime | None = None


# Secret Models
class SecretCreate(BaseModel):
    """Secret creation request."""

    key: str
    value: str


class SecretUpdate(BaseModel):
    """Secret update request."""

    value: str


class SecretResponse(BaseModel):
    """Secret metadata response (without value)."""

    key: str
    description: str | None = None
    created_at: datetime | None = None
    updated_at: datetime | None = None


class SecretValueResponse(BaseModel):
    """Secret value response (decrypted)."""

    key: str
    value: str
    description: str | None = None
    created_at: datetime | None = None
    updated_at: datetime | None = None


# Token Models
class TokenCreate(BaseModel):
    """Token creation request."""

    scope: str
    description: str | None = None
    password: str | None = None


class TokenResponse(BaseModel):
    """Token response."""

    id: str
    customer_id: str | None = None
    scope: str
    description: str | None = None
    token: str | None = None  # Only present on creation
    created_at: datetime | None = None
    expires_at: datetime | None = None


# Activity Models
class ActivityFilters(BaseModel):
    """Activity filters."""

    action: str | None = None
    method: str | None = None
    resource_id: str | None = None
    search: str | None = None
    start_date: datetime | None = None
    end_date: datetime | None = None


class ActivityResponse(BaseModel):
    """Activity response."""

    id: str | None = None
    action: str
    method: str | None = None
    resource_type: str | None = None
    resource_id: str | None = None
    customer_id: str | None = None
    project_id: str | None = None
    ip_address: str | None = None
    user_agent: str | None = None
    created_at: datetime | None = None
    metadata: dict[str, Any] | None = None



"""CLI configuration management."""

import base64
import json
import os
from pathlib import Path
from typing import Any

import yaml
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


class CLIConfig:
    """Manages CLI configuration and credentials."""

    def __init__(self):
        self.config_dir = Path.home() / ".vaulty"
        self.config_file = self.config_dir / "config.yaml"
        self.credentials_file = self.config_dir / "credentials.json"

        # Ensure config directory exists
        self.config_dir.mkdir(parents=True, exist_ok=True)

    def _get_encryption_key(self) -> bytes:
        """Get encryption key for credentials (derived from machine ID)."""
        # Use machine-specific identifier (hostname + user)
        machine_id = f"{os.getenv('HOSTNAME', 'localhost')}-{os.getenv('USER', 'user')}"

        # Derive key using PBKDF2 with SHA256 hash
        from cryptography.hazmat.primitives import hashes

        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=b"vaulty_cli_salt",  # Fixed salt for consistency
            iterations=100000,
            backend=default_backend(),
        )
        key = base64.urlsafe_b64encode(kdf.derive(machine_id.encode()))
        return key

    def load(self) -> dict[str, Any]:
        """Load configuration from file or environment."""
        config = {}

        # Load from file
        if self.config_file.exists():
            try:
                with open(self.config_file) as f:
                    config = yaml.safe_load(f) or {}
            except Exception:
                pass

        # Load credentials (encrypted) - includes stored base_url
        auth_info = self.get_auth_info()
        if auth_info:
            if auth_info.get("type") == "api_token":
                config["api_token"] = auth_info["token"]
                if auth_info.get("project"):
                    config["default_project"] = auth_info["project"]
                if auth_info.get("base_url"):
                    config["api_url"] = auth_info["base_url"]
            elif auth_info.get("type") == "jwt":
                config["jwt_token"] = auth_info["token"]
                if auth_info.get("base_url"):
                    config["api_url"] = auth_info["base_url"]

        # Override with environment variables (highest priority)
        if os.getenv("VAULTY_API_URL"):
            config["api_url"] = os.getenv("VAULTY_API_URL")
        if os.getenv("VAULTY_API_TOKEN"):
            config["api_token"] = os.getenv("VAULTY_API_TOKEN")
        if os.getenv("VAULTY_JWT_TOKEN"):
            config["jwt_token"] = os.getenv("VAULTY_JWT_TOKEN")

        # Default to production if no base_url set
        if not config.get("api_url"):
            config["api_url"] = "https://api.vaulty.com"

        return config

    def get_auth_info(self) -> dict[str, Any] | None:
        """Get authentication info from encrypted credentials file."""
        if not self.credentials_file.exists():
            return None

        try:
            key = self._get_encryption_key()
            fernet = Fernet(key)

            encrypted = self.credentials_file.read_bytes()
            decrypted = fernet.decrypt(encrypted)
            return json.loads(decrypted.decode())
        except Exception:
            return None

    def _validate_token(self, token: str) -> bool:
        """Validate token format.

        Args:
            token: Token to validate

        Returns:
            True if token appears valid
        """
        if not token or len(token) < 10:
            return False
        # Basic validation - tokens should start with 'vaulty_' or be JWT-like
        return token.startswith("vaulty_") or len(token.split(".")) >= 2

    def save_api_token(self, token: str, project: str | None = None, base_url: str | None = None):
        """Save API token and base URL (encrypted).

        Args:
            token: API token to save
            project: Optional project name
            base_url: Optional base URL

        Raises:
            ValueError: If token format is invalid
        """
        if not self._validate_token(token):
            raise ValueError(
                "Invalid token format. Token should start with 'vaulty_' or be a valid JWT."
            )

        key = self._get_encryption_key()
        fernet = Fernet(key)

        credentials = {
            "type": "api_token",
            "token": token,
            "project": project,
            "base_url": base_url,
        }

        encrypted = fernet.encrypt(json.dumps(credentials).encode())
        self.credentials_file.write_bytes(encrypted)
        self.credentials_file.chmod(0o600)

    def save_jwt_token(self, token: str, email: str | None = None, base_url: str | None = None):
        """Save JWT token and base URL (encrypted).

        Args:
            token: JWT token to save
            email: Optional email address
            base_url: Optional base URL

        Raises:
            ValueError: If token format is invalid
        """
        if not self._validate_token(token):
            raise ValueError("Invalid JWT token format.")

        key = self._get_encryption_key()
        fernet = Fernet(key)

        credentials = {
            "type": "jwt",
            "token": token,
            "email": email,
            "base_url": base_url,
        }

        encrypted = fernet.encrypt(json.dumps(credentials).encode())
        self.credentials_file.write_bytes(encrypted)
        self.credentials_file.chmod(0o600)

    def clear_credentials(self):
        """Clear stored credentials."""
        if self.credentials_file.exists():
            self.credentials_file.unlink()



"""Customer resource client."""

from ..http import HTTPClient
from ..models import (
    CustomerResponse,
    CustomerSettingsResponse,
)
from ..retry import RetryConfig, retry_with_backoff


class CustomerResource:
    """Client for customer management operations."""

    def __init__(self, http_client: HTTPClient, retry_config: RetryConfig | None = None):
        self.http_client = http_client
        self.retry_config = retry_config

    async def register(self, email: str, password: str) -> CustomerResponse:
        """Register a new customer.

        Args:
            email: Customer email
            password: Customer password (min 8 characters)

        Returns:
            CustomerResponse with customer data
        """

        async def _register():
            response = await self.http_client.post(
                "/api/v1/customers/register", json={"email": email, "password": password}
            )
            return CustomerResponse(**response.json())

        return await retry_with_backoff(_register, self.retry_config)

    async def login(self, email: str, password: str) -> dict:
        """Login and get JWT token.

        Args:
            email: Customer email
            password: Customer password

        Returns:
            Token response with access_token
        """

        async def _login():
            response = await self.http_client.post(
                "/api/v1/customers/login", json={"email": email, "password": password}
            )
            return response.json()

        return await retry_with_backoff(_login, self.retry_config)

    async def get_current(self) -> CustomerResponse:
        """Get current customer info.

        Returns:
            CustomerResponse with current customer data
        """

        async def _get_current():
            response = await self.http_client.get("/api/v1/customers/me")
            return CustomerResponse(**response.json())

        return await retry_with_backoff(_get_current, self.retry_config)

    async def update_settings(
        self,
        rate_limit_enabled: bool | None = None,
        rate_limit_requests_per_minute: int | None = None,
        rate_limit_auth_attempts_per_minute: int | None = None,
        cache_ttl_customer: int | None = None,
        cache_ttl_project: int | None = None,
        cache_ttl_token: int | None = None,
        cache_ttl_dek: int | None = None,
    ) -> CustomerSettingsResponse:
        """Update customer settings.

        Args:
            rate_limit_enabled: Enable/disable rate limiting
            rate_limit_requests_per_minute: Rate limit for API requests
            rate_limit_auth_attempts_per_minute: Rate limit for auth endpoints
            cache_ttl_customer: Cache TTL for customer data
            cache_ttl_project: Cache TTL for project data
            cache_ttl_token: Cache TTL for token data
            cache_ttl_dek: Cache TTL for DEK data

        Returns:
            CustomerSettingsResponse with updated settings
        """
        data = {}
        if rate_limit_enabled is not None:
            data["rate_limit_enabled"] = rate_limit_enabled
        if rate_limit_requests_per_minute is not None:
            data["rate_limit_requests_per_minute"] = rate_limit_requests_per_minute
        if rate_limit_auth_attempts_per_minute is not None:
            data["rate_limit_auth_attempts_per_minute"] = rate_limit_auth_attempts_per_minute
        if cache_ttl_customer is not None:
            data["cache_ttl_customer"] = cache_ttl_customer
        if cache_ttl_project is not None:
            data["cache_ttl_project"] = cache_ttl_project
        if cache_ttl_token is not None:
            data["cache_ttl_token"] = cache_ttl_token
        if cache_ttl_dek is not None:
            data["cache_ttl_dek"] = cache_ttl_dek

        async def _update_settings():
            response = await self.http_client.patch("/api/v1/customers/settings", json=data)
            return CustomerSettingsResponse(**response.json())

        return await retry_with_backoff(_update_settings, self.retry_config)

    async def get_settings(self) -> CustomerSettingsResponse:
        """Get customer settings.

        Returns:
            CustomerSettingsResponse with current settings
        """

        async def _get_settings():
            response = await self.http_client.get("/api/v1/customers/settings")
            return CustomerSettingsResponse(**response.json())

        return await retry_with_backoff(_get_settings, self.retry_config)



"""Project resource client."""

import urllib.parse

from ..http import HTTPClient
from ..models import (
    PaginatedResponse,
    ProjectResponse,
)
from ..retry import RetryConfig, retry_with_backoff


class ProjectResource:
    """Client for project management operations."""

    def __init__(self, http_client: HTTPClient, retry_config: RetryConfig | None = None):
        self.http_client = http_client
        self.retry_config = retry_config

    async def create(self, name: str, description: str | None = None) -> ProjectResponse:
        """Create a new project.

        Args:
            name: Project name
            description: Optional project description

        Returns:
            ProjectResponse with created project data
        """

        async def _create():
            response = await self.http_client.post(
                "/api/v1/projects", json={"name": name, "description": description}
            )
            return ProjectResponse(**response.json())

        return await retry_with_backoff(_create, self.retry_config)

    async def list(self, page: int = 1, page_size: int = 50) -> PaginatedResponse[ProjectResponse]:
        """List projects with pagination.

        Args:
            page: Page number (1-indexed)
            page_size: Number of items per page (1-100)

        Returns:
            PaginatedResponse with projects
        """

        async def _list():
            response = await self.http_client.get(
                "/api/v1/projects", params={"page": page, "page_size": page_size}
            )
            data = response.json()
            return PaginatedResponse[ProjectResponse](
                items=[ProjectResponse(**item) for item in data["items"]],
                total=data["total"],
                page=data["page"],
                page_size=data["page_size"],
                total_pages=data["total_pages"],
                has_next=data["has_next"],
                has_previous=data["has_previous"],
            )

        return await retry_with_backoff(_list, self.retry_config)

    async def get(self, name: str) -> ProjectResponse:
        """Get project by name.

        Args:
            name: Project name

        Returns:
            ProjectResponse with project data
        """

        async def _get():
            # URL encode project name
            encoded_name = urllib.parse.quote(name, safe="")
            response = await self.http_client.get(f"/api/v1/projects/{encoded_name}")
            return ProjectResponse(**response.json())

        return await retry_with_backoff(_get, self.retry_config)

    async def update(self, name: str, description: str | None = None) -> ProjectResponse:
        """Update project.

        Args:
            name: Project name
            description: Optional project description

        Returns:
            ProjectResponse with updated project data
        """

        async def _update():
            encoded_name = urllib.parse.quote(name, safe="")
            response = await self.http_client.patch(
                f"/api/v1/projects/{encoded_name}",
                json={"description": description} if description else {},
            )
            return ProjectResponse(**response.json())

        return await retry_with_backoff(_update, self.retry_config)

    async def delete(self, name: str) -> None:
        """Delete project.

        Args:
            name: Project name
        """

        async def _delete():
            encoded_name = urllib.parse.quote(name, safe="")
            await self.http_client.delete(f"/api/v1/projects/{encoded_name}")

        await retry_with_backoff(_delete, self.retry_config)



"""Vaulty Python SDK."""

from importlib.metadata import PackageNotFoundError, version

from .client import VaultyClient
from .exceptions import (
    VaultyAPIError,
    VaultyAuthenticationError,
    VaultyAuthorizationError,
    VaultyError,
    VaultyNotFoundError,
    VaultyRateLimitError,
    VaultyValidationError,
)

__all__ = [
    "VaultyAPIError",
    "VaultyAuthenticationError",
    "VaultyAuthorizationError",
    "VaultyClient",
    "VaultyError",
    "VaultyNotFoundError",
    "VaultyRateLimitError",
    "VaultyValidationError",
]

try:
    __version__ = version("vaulty-client")
except PackageNotFoundError:
    # Package not installed, use version from pyproject.toml
    __version__ = "0.1.1"



"""Activity resource client."""

from datetime import datetime

from ..http import HTTPClient
from ..models import (
    ActivityResponse,
    PaginatedResponse,
)
from ..retry import RetryConfig, retry_with_backoff


class ActivityResource:
    """Client for activity management operations."""

    def __init__(self, http_client: HTTPClient, retry_config: RetryConfig | None = None):
        self.http_client = http_client
        self.retry_config = retry_config

    async def list(
        self,
        page: int = 1,
        page_size: int = 50,
        action: str | None = None,
        method: str | None = None,
        resource_id: str | None = None,
        search: str | None = None,
        start_date: datetime | None = None,
        end_date: datetime | None = None,
    ) -> PaginatedResponse[ActivityResponse]:
        """List activities with filters and pagination.

        Args:
            page: Page number (1-indexed)
            page_size: Number of items per page (1-100)
            action: Filter by action (e.g., "create_secret")
            method: Filter by HTTP method (e.g., "POST")
            resource_id: Filter by resource ID
            search: Search term
            start_date: Filter activities after this date
            end_date: Filter activities before this date

        Returns:
            PaginatedResponse with activities
        """

        async def _list():
            params = {"page": page, "page_size": page_size}
            if action:
                params["action"] = action
            if method:
                params["method"] = method
            if resource_id:
                params["resource_id"] = resource_id
            if search:
                params["search"] = search
            if start_date:
                params["start_date"] = start_date.isoformat()
            if end_date:
                params["end_date"] = end_date.isoformat()

            response = await self.http_client.get("/api/v1/activities", params=params)
            data = response.json()

            # Parse datetime strings back to datetime objects
            items = []
            for item in data["items"]:
                # Parse created_at
                if isinstance(item.get("created_at"), str):
                    item["created_at"] = datetime.fromisoformat(
                        item["created_at"].replace("Z", "+00:00")
                    )
                items.append(ActivityResponse(**item))

            return PaginatedResponse[ActivityResponse](
                items=items,
                total=data["total"],
                page=data["page"],
                page_size=data["page_size"],
                total_pages=data["total_pages"],
                has_next=data["has_next"],
                has_previous=data["has_previous"],
            )

        return await retry_with_backoff(_list, self.retry_config)



"""Retry logic with exponential backoff."""

import asyncio
import random
from collections.abc import Callable
from typing import TypeVar

from .exceptions import VaultyAPIError, VaultyRateLimitError
from .logging import get_logger

logger = get_logger(__name__)
T = TypeVar("T")


class RetryConfig:
    """Configuration for retry behavior."""

    def __init__(
        self,
        max_retries: int = 3,
        initial_delay: float = 1.0,
        max_delay: float = 60.0,
        backoff_factor: float = 2.0,
        jitter: bool = True,
    ):
        self.max_retries = max_retries
        self.initial_delay = initial_delay
        self.max_delay = max_delay
        self.backoff_factor = backoff_factor
        self.jitter = jitter


async def retry_with_backoff(
    func: Callable[..., T], config: RetryConfig = None, *args, **kwargs
) -> T:
    """Retry a function with exponential backoff.

    Args:
        func: Async function to retry
        config: Retry configuration
        *args: Positional arguments for func
        **kwargs: Keyword arguments for func

    Returns:
        Result of func

    Raises:
        VaultyAPIError: If all retries fail
    """
    if config is None:
        config = RetryConfig()

    last_exception = None

    for attempt in range(config.max_retries + 1):
        try:
            return await func(*args, **kwargs)
        except VaultyRateLimitError as e:
            # Handle rate limit errors specially
            last_exception = e
            if attempt < config.max_retries:
                # Use retry_after if provided, otherwise calculate delay
                delay = (
                    e.retry_after
                    if e.retry_after
                    else config.initial_delay * (config.backoff_factor**attempt)
                )
                delay = min(delay, config.max_delay)
                if config.jitter:
                    delay += random.uniform(0, delay * 0.1)
                logger.info(
                    f"Rate limit hit, retrying in {delay:.2f}s (attempt {attempt + 1}/{config.max_retries + 1})",
                    extra={
                        "attempt": attempt + 1,
                        "max_retries": config.max_retries,
                        "delay": delay,
                    },
                )
                await asyncio.sleep(delay)
            else:
                logger.error(f"Rate limit retry exhausted after {config.max_retries + 1} attempts")
                raise
        except VaultyAPIError as e:
            # Retry on 5xx errors, don't retry on 4xx (except rate limit)
            if e.status_code >= 500 and attempt < config.max_retries:
                last_exception = e
                delay = config.initial_delay * (config.backoff_factor**attempt)
                delay = min(delay, config.max_delay)
                if config.jitter:
                    delay += random.uniform(0, delay * 0.1)
                logger.warning(
                    f"Server error {e.status_code}, retrying in {delay:.2f}s (attempt {attempt + 1}/{config.max_retries + 1})",
                    extra={"attempt": attempt + 1, "status_code": e.status_code, "delay": delay},
                )
                await asyncio.sleep(delay)
            else:
                raise
        except Exception as e:
            # Retry on network errors, etc.
            last_exception = e
            if attempt < config.max_retries:
                delay = config.initial_delay * (config.backoff_factor**attempt)
                delay = min(delay, config.max_delay)
                if config.jitter:
                    delay += random.uniform(0, delay * 0.1)
                logger.warning(
                    f"Request failed: {type(e).__name__}, retrying in {delay:.2f}s (attempt {attempt + 1}/{config.max_retries + 1})",
                    exc_info=True,
                    extra={"attempt": attempt + 1, "error_type": type(e).__name__, "delay": delay},
                )
                await asyncio.sleep(delay)
            else:
                logger.error(
                    f"Retry exhausted after {config.max_retries + 1} attempts", exc_info=True
                )
                raise

    if last_exception:
        raise last_exception



"""Health check commands."""

import sys

import click

from ... import VaultyClient
from ...cli.output import OutputFormatter
from ...cli.utils import run_async


@click.group()
def health_group():
    """Health check commands."""


@health_group.command()
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def health(format, base_url):
    """Health check."""
    try:
        import os

        api_base_url = base_url or os.getenv("VAULTY_API_URL", "https://api.vaulty.com")
        client = VaultyClient(base_url=api_base_url)
        health_status = run_async(client.health.check())

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(health_status))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@health_group.command("ready")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def ready(format, base_url):
    """Readiness check."""
    try:
        import os

        api_base_url = base_url or os.getenv("VAULTY_API_URL", "https://api.vaulty.com")
        client = VaultyClient(base_url=api_base_url)
        ready_status = run_async(client.health.ready())

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(ready_status))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@health_group.command("live")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def live(format, base_url):
    """Liveness check."""
    try:
        import os

        api_base_url = base_url or os.getenv("VAULTY_API_URL", "https://api.vaulty.com")
        client = VaultyClient(base_url=api_base_url)
        live_status = run_async(client.health.live())

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(live_status))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Custom exceptions for Vaulty SDK."""


class VaultyError(Exception):
    """Base exception for all Vaulty errors."""


class VaultyAPIError(VaultyError):
    """API returned an error response."""

    def __init__(self, message: str, status_code: int, detail: str | None = None):
        self.status_code = status_code
        self.detail = detail
        super().__init__(message)


class VaultyAuthenticationError(VaultyAPIError):
    """Authentication failed."""


class VaultyAuthorizationError(VaultyAPIError):
    """Insufficient permissions."""


class VaultyNotFoundError(VaultyAPIError):
    """Resource not found."""


class VaultyValidationError(VaultyAPIError):
    """Request validation failed."""


class VaultyRateLimitError(VaultyAPIError):
    """Rate limit exceeded."""

    def __init__(
        self,
        message: str,
        status_code: int,
        detail: str | None = None,
        retry_after: int | None = None,
    ):
        self.retry_after = retry_after
        super().__init__(message, status_code, detail)



"""Authentication handling for Vaulty SDK."""

from .http import HTTPClient
from .logging import get_logger

logger = get_logger(__name__)


class AuthHandler:
    """Handles authentication for Vaulty API."""

    def __init__(self, http_client: HTTPClient):
        self.http_client = http_client
        self._jwt_token: str | None = None

    async def login(self, email: str, password: str) -> dict:
        """Login with email/password and get JWT token.

        Args:
            email: Customer email
            password: Customer password

        Returns:
            Token response with access_token
        """
        logger.info(f"Attempting login for user: {email}")
        try:
            response = await self.http_client.post(
                "/api/customers/login", json={"email": email, "password": password}
            )
            data = response.json()
            self._jwt_token = data.get("access_token")

            # Update HTTP client with new JWT token
            if self._jwt_token:
                logger.info("Login successful, JWT token obtained")
                self.http_client.jwt_token = self._jwt_token
                self.http_client.auth_header = f"Bearer {self._jwt_token}"
                # Recreate client to update headers
                await self.http_client.close()
            else:
                logger.warning("Login response missing access_token")

            return data
        except Exception:
            logger.error(f"Login failed for user: {email}", exc_info=True)
            raise

    @property
    def jwt_token(self) -> str | None:
        """Get current JWT token."""
        return self._jwt_token

    @jwt_token.setter
    def jwt_token(self, token: str):
        """Set JWT token."""
        self._jwt_token = token
        self.http_client.jwt_token = token
        self.http_client.auth_header = f"Bearer {token}"



"""Activity commands."""

import sys
from datetime import datetime

import click

from ...cli.output import OutputFormatter
from ...cli.utils import get_client, run_async


@click.group()
def activities_group():
    """Activity management commands."""


@activities_group.command("list")
@click.option("--action", help="Filter by action (e.g., create_secret)")
@click.option("--method", help="Filter by HTTP method (e.g., POST)")
@click.option("--resource-id", help="Filter by resource ID")
@click.option("--search", help="Search term")
@click.option("--start-date", help="Filter activities after this date (YYYY-MM-DD)")
@click.option("--end-date", help="Filter activities before this date (YYYY-MM-DD)")
@click.option("--page", default=1, help="Page number")
@click.option("--page-size", default=50, help="Items per page")
@click.option(
    "--format", "-f", default="table", type=click.Choice(["json", "yaml", "plain", "table"])
)
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def list_activities(
    action,
    method,
    resource_id,
    search,
    start_date,
    end_date,
    page,
    page_size,
    format,
    token,
    base_url,
):
    """List activities with filters."""
    try:
        client = get_client(token=token, base_url=base_url)

        # Parse dates
        start_dt = None
        end_dt = None
        if start_date:
            start_dt = datetime.fromisoformat(start_date)
        if end_date:
            end_dt = datetime.fromisoformat(end_date)

        result = run_async(
            client.activities.list(
                page=page,
                page_size=page_size,
                action=action,
                method=method,
                resource_id=resource_id,
                search=search,
                start_date=start_dt,
                end_date=end_dt,
            )
        )

        formatter = OutputFormatter(format=format)
        output = formatter.format_output(
            {
                "items": [item.dict() for item in result.items],
                "total": result.total,
                "page": result.page,
                "page_size": result.page_size,
                "total_pages": result.total_pages,
                "has_next": result.has_next,
                "has_previous": result.has_previous,
            }
        )
        click.echo(output)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Customer commands."""

import sys

import click

from ...cli.output import OutputFormatter
from ...cli.utils import get_client, run_async
from ...client import VaultyClient


@click.group()
def customers_group():
    """Customer management commands."""


@customers_group.command("register")
@click.option("--email", "-e", required=True, help="Email address")
@click.option("--password", "-p", required=True, help="Password (min 8 characters)")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def register_customer(email, password, format, base_url):
    """Register new customer."""
    try:
        import os

        api_base_url = base_url or os.getenv("VAULTY_API_URL", "https://api.vaulty.com")
        client = VaultyClient(base_url=api_base_url)
        customer = run_async(client.customers.register(email=email, password=password))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(customer.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@customers_group.command("get")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def get_customer(format, token, base_url):
    """Get current customer info."""
    try:
        client = get_client(token=token, base_url=base_url)
        customer = run_async(client.customers.get_current())

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(customer.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@customers_group.group("settings")
def settings_group():
    """Customer settings commands."""


@settings_group.command("get")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def get_settings(format, token, base_url):
    """Get customer settings."""
    try:
        client = get_client(token=token, base_url=base_url)
        settings = run_async(client.customers.get_settings())

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(settings.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@settings_group.command("update")
@click.option("--rate-limit-enabled", type=bool, help="Enable/disable rate limiting")
@click.option("--rate-limit-requests-per-minute", type=int, help="Rate limit for API requests")
@click.option(
    "--rate-limit-auth-attempts-per-minute", type=int, help="Rate limit for auth endpoints"
)
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def update_settings(
    rate_limit_enabled,
    rate_limit_requests_per_minute,
    rate_limit_auth_attempts_per_minute,
    format,
    token,
    base_url,
):
    """Update customer settings."""
    try:
        client = get_client(token=token, base_url=base_url)
        settings = run_async(
            client.customers.update_settings(
                rate_limit_enabled=rate_limit_enabled,
                rate_limit_requests_per_minute=rate_limit_requests_per_minute,
                rate_limit_auth_attempts_per_minute=rate_limit_auth_attempts_per_minute,
            )
        )

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(settings.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Resource clients for Vaulty API."""

from .activities import ActivityResource
from .customers import CustomerResource
from .health import HealthResource
from .projects import ProjectResource
from .secrets import SecretResource
from .tokens import TokenResource

__all__ = [
    "ActivityResource",
    "CustomerResource",
    "HealthResource",
    "ProjectResource",
    "SecretResource",
    "TokenResource",
]



"""CLI entry point."""

import sys

import click

from .. import __version__
from .commands import activities, auth, customers, health, projects, secrets, tokens


@click.group()
@click.version_option(version=__version__)
def cli():
    """Vaulty CLI - Manage secrets, projects, and tokens."""


# Add command groups
cli.add_command(auth.auth_group)
cli.add_command(projects.projects_group)
cli.add_command(secrets.secrets_group)
cli.add_command(tokens.tokens_group)
cli.add_command(activities.activities_group)
cli.add_command(customers.customers_group)
cli.add_command(health.health_group)


# Convenience shortcuts for common operations
@cli.command("login")
@click.argument("token", required=False)
@click.option("--email", "-e", help="Email address (for JWT login)")
@click.option("--password", "-p", help="Password (prompted if not provided)")
@click.option("--token", "-t", "token_flag", help="API token (alternative to positional)")
@click.option("--token-env", help="Environment variable containing API token")
@click.option("--token-file", help="File containing API token")
@click.option("--project", help="Project name (for project-scoped tokens)")
@click.option("--base-url", "-u", help="Base URL for API (e.g., http://localhost:8000)")
def login_shortcut(token, email, password, token_flag, token_env, token_file, project, base_url):
    """Login and store credentials (shortcut for 'auth login').

    Examples:
        vaulty login TOKEN
        vaulty login TOKEN --base-url http://localhost:8000
        vaulty login --email user@example.com
    """
    import os

    from .. import VaultyAPIError, VaultyClient
    from .config import CLIConfig
    from .utils import run_async

    config = CLIConfig()

    # Determine base URL (priority: --base-url > env var > default)
    if base_url:
        api_base_url = base_url
    elif os.getenv("VAULTY_API_URL"):
        api_base_url = os.getenv("VAULTY_API_URL")
    else:
        api_base_url = "https://api.vaulty.com"

    # Determine authentication method
    api_token = None

    if token:  # Positional argument (default)
        api_token = token
    elif token_flag:  # --token flag
        api_token = token_flag
    elif token_env:
        api_token = os.getenv(token_env)
        if not api_token:
            click.echo(f"Error: Environment variable {token_env} not set", err=True)
            sys.exit(1)
    elif token_file:
        try:
            with open(token_file) as f:
                api_token = f.read().strip()
        except FileNotFoundError:
            click.echo(f"Error: Token file not found: {token_file}", err=True)
            sys.exit(1)
    elif email:
        if not password:
            password = click.prompt("Password", hide_input=True)

        # Login with email/password to get JWT
        try:
            client = VaultyClient(base_url=api_base_url)
            token_response = run_async(client.customers.login(email, password))
            # Store JWT token and base URL
            config.save_jwt_token(
                token_response["access_token"], email=email, base_url=api_base_url
            )
            click.echo("Logged in successfully!")
            click.echo(f"Base URL: {api_base_url}")
            return
        except VaultyAPIError as e:
            click.echo(f"Error: {e.detail}", err=True)
            sys.exit(1)
    else:
        click.echo("Error: Must provide token, --email, --token-env, or --token-file", err=True)
        click.echo("Usage: vaulty login <token> [--base-url <url>]", err=True)
        sys.exit(2)

    # Store API token and base URL
    config.save_api_token(api_token, project=project, base_url=api_base_url)
    click.echo("Logged in successfully!")
    click.echo(f"Base URL: {api_base_url}")

    # Verify token and show info
    try:
        client = VaultyClient(api_token=api_token, base_url=api_base_url)
        customer = run_async(client.customers.get_current())
        click.echo(f"Authenticated as: {customer.email}")
        if project:
            click.echo(f"Project scope: {project}")
    except VaultyAPIError as e:
        click.echo(f"Warning: Could not verify token: {e.detail}", err=True)


@cli.command("logout")
def logout_shortcut():
    """Logout and clear stored credentials (shortcut for 'auth logout').

    Examples:
        vaulty logout
    """
    from .config import CLIConfig

    config = CLIConfig()
    config.clear_credentials()
    click.echo("Logged out successfully!")


# Convenience shortcuts for common operations
@cli.command("get")
@click.argument("key")
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--format", "-f", default=None, type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def get_shortcut(key, project, format, token, base_url):
    """Get secret value (shortcut for 'secrets get').

    Examples:
        vaulty get API_KEY
        vaulty get API_KEY --project my-project
    """
    from .commands.secrets import get_secret

    get_secret(key, project, format, token, base_url)


@cli.command("g")
@click.argument("key")
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--format", "-f", default=None, type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def get_ultra_short(key, project, format, token, base_url):
    """Get secret value (ultra-short alias: 'g').

    Examples:
        vaulty g API_KEY
        vaulty g API_KEY --project my-project
    """
    from .commands.secrets import get_secret

    get_secret(key, project, format, token, base_url)


@cli.command("set")
@click.argument("key")
@click.argument("value", required=False)
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--prompt", is_flag=True, help="Prompt for value (hidden input)")
@click.option("--stdin", is_flag=True, help="Read value from stdin")
@click.option("--file", help="Read value from file")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def set_shortcut(key, value, project, prompt, stdin, file, format, token, base_url):
    """Set secret value (shortcut for 'secrets create').

    Examples:
        vaulty set API_KEY "secret-value"
        vaulty set API_KEY --prompt
        vaulty set API_KEY --stdin
    """
    import sys

    from .output import OutputFormatter
    from .utils import get_client, get_project_from_token_scope, run_async

    try:
        # Determine secret value
        if stdin:
            import sys as sys_module

            value = sys_module.stdin.read().strip()
        elif file:
            with open(file) as f:
                value = f.read().strip()
        elif prompt:
            value = click.prompt("Value", hide_input=True)
        elif not value:
            click.echo("Error: Must provide value, --prompt, --stdin, or --file", err=True)
            sys.exit(2)

        if not project:
            import os

            project = os.getenv("VAULTY_PROJECT")
            if not project:
                # Try to infer from token scope (for project-scoped tokens)
                client = get_client(token=token, base_url=base_url)
                project_info = run_async(get_project_from_token_scope(client))
                if not project_info:
                    click.echo(
                        "Error: --project is required for full-scope tokens. For project-scoped tokens, project is auto-detected.",
                        err=True,
                    )
                    sys.exit(2)
                # Use project name if available, otherwise use project ID
                project = project_info.get("name") or project_info.get("id")

        client = get_client(token=token, base_url=base_url)
        secret = run_async(client.secrets.create(project_name=project, key=key, value=value))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(secret.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@cli.command("delete")
@click.argument("key")
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def delete_shortcut(key, project, token, base_url):
    """Delete secret (shortcut for 'secrets delete').

    Examples:
        vaulty delete API_KEY
        vaulty delete API_KEY --project my-project
    """
    import sys

    from .utils import get_client, get_project_from_token_scope, run_async

    try:
        if not project:
            import os

            project = os.getenv("VAULTY_PROJECT")
            if not project:
                # Try to infer from token scope (for project-scoped tokens)
                client = get_client(token=token, base_url=base_url)
                project_info = run_async(get_project_from_token_scope(client))
                if not project_info:
                    click.echo(
                        "Error: --project is required for full-scope tokens. For project-scoped tokens, project is auto-detected.",
                        err=True,
                    )
                    sys.exit(2)
                # Use project name if available, otherwise use project ID
                project = project_info.get("name") or project_info.get("id")

        client = get_client(token=token, base_url=base_url)
        run_async(client.secrets.delete(project_name=project, key=key))
        click.echo("Secret deleted successfully!")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@cli.command("list")
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--page", default=1, help="Page number")
@click.option("--page-size", default=50, help="Items per page")
@click.option("--format", "-f", default=None, type=click.Choice(["json", "yaml", "plain", "table"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def list_shortcut(project, page, page_size, format, token, base_url):
    """List secrets (shortcut for 'secrets list').

    Examples:
        vaulty list
        vaulty list --project my-project
    """
    # Import the actual implementation logic
    import sys

    from .output import OutputFormatter
    from .utils import detect_cicd, get_client, get_project_from_token_scope, run_async

    # Default format based on CI/CD detection
    if format is None:
        format = "plain" if detect_cicd() else "table"

    try:
        client = get_client(token=token, base_url=base_url)

        # Determine project name
        if not project:
            import os

            project = os.getenv("VAULTY_PROJECT")
            if not project:
                # Try to infer from token scope (for project-scoped tokens)
                project_info = run_async(get_project_from_token_scope(client))
                if not project_info:
                    click.echo(
                        "Error: --project is required for full-scope tokens. For project-scoped tokens, project is auto-detected.",
                        err=True,
                    )
                    sys.exit(2)
                # Use project name if available, otherwise use project ID
                project = project_info.get("name") or project_info.get("id")

        result = run_async(
            client.secrets.list(project_name=project, page=page, page_size=page_size)
        )

        formatter = OutputFormatter(format=format)
        output = formatter.format_output(
            {
                "items": [item.dict() for item in result.items],
                "total": result.total,
                "page": result.page,
                "page_size": result.page_size,
                "total_pages": result.total_pages,
                "has_next": result.has_next,
                "has_previous": result.has_previous,
            }
        )
        click.echo(output)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@cli.command("update")
@click.argument("key")
@click.argument("value", required=False)
@click.option(
    "--project", "-p", help="Project name (required for full scope, optional for project-scoped)"
)
@click.option("--prompt", is_flag=True, help="Prompt for value (hidden input)")
@click.option("--stdin", is_flag=True, help="Read value from stdin")
@click.option("--file", help="Read value from file")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def update_shortcut(key, value, project, prompt, stdin, file, format, token, base_url):
    """Update secret value (shortcut for 'secrets update').

    Examples:
        vaulty update API_KEY "new-value"
        vaulty update API_KEY --prompt
    """
    import sys

    from .output import OutputFormatter
    from .utils import get_client, get_project_from_token_scope, run_async

    try:
        # Determine secret value
        if stdin:
            import sys as sys_module

            value = sys_module.stdin.read().strip()
        elif file:
            with open(file) as f:
                value = f.read().strip()
        elif prompt:
            value = click.prompt("Value", hide_input=True)
        elif not value:
            click.echo("Error: Must provide value, --prompt, --stdin, or --file", err=True)
            sys.exit(2)

        if not project:
            import os

            project = os.getenv("VAULTY_PROJECT")
            if not project:
                # Try to infer from token scope (for project-scoped tokens)
                client = get_client(token=token, base_url=base_url)
                project_info = run_async(get_project_from_token_scope(client))
                if not project_info:
                    click.echo(
                        "Error: --project is required for full-scope tokens. For project-scoped tokens, project is auto-detected.",
                        err=True,
                    )
                    sys.exit(2)
                # Use project name if available, otherwise use project ID
                project = project_info.get("name") or project_info.get("id")

        client = get_client(token=token, base_url=base_url)
        secret = run_async(client.secrets.update(project_name=project, key=key, value=value))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(secret.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


def main():
    """Main entry point for CLI."""
    cli()


if __name__ == "__main__":
    main()



"""Project commands."""

import sys

import click

from ...cli.output import OutputFormatter
from ...cli.utils import get_client, run_async


@click.group()
def projects_group():
    """Project management commands."""


@projects_group.command("list")
@click.option("--page", default=1, help="Page number")
@click.option("--page-size", default=50, help="Items per page")
@click.option(
    "--format", "-f", default="table", type=click.Choice(["json", "yaml", "plain", "table"])
)
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def list_projects(page, page_size, format, token, base_url):
    """List all projects."""
    try:
        client = get_client(token=token, base_url=base_url)
        result = run_async(client.projects.list(page=page, page_size=page_size))

        formatter = OutputFormatter(format=format)
        output = formatter.format_output(
            {
                "items": [item.dict() for item in result.items],
                "total": result.total,
                "page": result.page,
                "page_size": result.page_size,
                "total_pages": result.total_pages,
                "has_next": result.has_next,
                "has_previous": result.has_previous,
            }
        )
        click.echo(output)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@projects_group.command("get")
@click.argument("name")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def get_project(name, format, token, base_url):
    """Get project details."""
    try:
        client = get_client(token=token, base_url=base_url)
        project = run_async(client.projects.get(name))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(project.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@projects_group.command("create")
@click.argument("name")
@click.option("--description", help="Project description")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def create_project(name, description, format, token, base_url):
    """Create project."""
    try:
        client = get_client(token=token, base_url=base_url)
        project = run_async(client.projects.create(name=name, description=description))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(project.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@projects_group.command("update")
@click.argument("name")
@click.option("--description", help="Project description")
@click.option("--format", "-f", default="json", type=click.Choice(["json", "yaml", "plain"]))
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def update_project(name, description, format, token, base_url):
    """Update project."""
    try:
        client = get_client(token=token, base_url=base_url)
        project = run_async(client.projects.update(name=name, description=description))

        formatter = OutputFormatter(format=format)
        click.echo(formatter.format_output(project.dict()))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)


@projects_group.command("delete")
@click.argument("name")
@click.option("--token", "-t", help="API token (overrides stored credentials)")
@click.option("--base-url", "-u", help="Base URL (overrides stored/configured URL)")
def delete_project(name, token, base_url):
    """Delete project."""
    try:
        client = get_client(token=token, base_url=base_url)
        run_async(client.projects.delete(name))
        click.echo("Project deleted successfully!")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)



"""Output formatting for CLI."""

import json
from typing import Any

import yaml
from rich.console import Console
from rich.table import Table


class OutputFormatter:
    """Formats output for CLI commands."""

    def __init__(self, format: str = "plain"):
        self.format = format
        self.console = Console()

    def format_output(self, data: Any) -> str:
        """Format output based on format type.

        Args:
            data: Data to format

        Returns:
            Formatted string
        """
        if self.format == "json":
            return json.dumps(data, indent=2, default=str)
        if self.format == "yaml":
            return yaml.dump(data, default_flow_style=False, default_style=None)
        if self.format == "table":
            return self._format_table(data)
        # plain
        return self._format_plain(data)

    def _format_plain(self, data: Any) -> str:
        """Format as plain text."""
        if isinstance(data, dict):
            # For secret value responses, return just the value
            if "value" in data:
                return str(data["value"])
            # For list responses, return one item per line
            if "items" in data:
                if isinstance(data["items"], list) and len(data["items"]) > 0:
                    if isinstance(data["items"][0], dict):
                        # Extract key field if available
                        if "key" in data["items"][0]:
                            return "\n".join(str(item["key"]) for item in data["items"])
                        if "name" in data["items"][0]:
                            return "\n".join(str(item["name"]) for item in data["items"])
            return str(data)
        if isinstance(data, list):
            return "\n".join(str(item) for item in data)
        return str(data)

    def _format_table(self, data: Any) -> str:
        """Format as table."""
        if isinstance(data, dict) and "items" in data:
            items = data["items"]
            if not items:
                return "No items found."

            # Determine columns from first item
            if isinstance(items[0], dict):
                columns = list(items[0].keys())
                table = Table()

                for col in columns:
                    table.add_column(col.replace("_", " ").title())

                for item in items:
                    table.add_row(*[str(item.get(col, "")) for col in columns])

                console = Console()
                with console.capture() as capture:
                    console.print(table)
                return capture.get()

        return self._format_plain(data)

    def format_env(self, data: Any, prefix: str = "VAULTY_SECRET_") -> str:
        """Format as environment variables.

        Args:
            data: Data to format
            prefix: Prefix for environment variable names

        Returns:
            Environment variable export statements
        """
        exports = []

        if isinstance(data, dict) and "items" in data:
            # List of secrets
            for item in data["items"]:
                if isinstance(item, dict) and "key" in item and "value" in item:
                    key = item["key"]
                    value = item["value"]
                    var_name = f"{prefix}{key}"
                    exports.append(f"export {var_name}={value}")
        elif isinstance(data, dict) and "key" in data and "value" in data:
            # Single secret
            key = data["key"]
            value = data["value"]
            var_name = f"{prefix}{key}"
            exports.append(f"export {var_name}={value}")

        return "\n".join(exports)



"""CLI commands."""

from . import activities, auth, customers, health, projects, secrets, tokens

__all__ = [
    "activities",
    "auth",
    "customers",
    "health",
    "projects",
    "secrets",
    "tokens",
]



"""Health check resource client."""

from typing import Any

from ..http import HTTPClient
from ..retry import RetryConfig, retry_with_backoff


class HealthResource:
    """Client for health check operations."""

    def __init__(self, http_client: HTTPClient, retry_config: RetryConfig = None):
        self.http_client = http_client
        self.retry_config = retry_config

    async def check(self) -> dict[str, Any]:
        """Health check.

        Returns:
            Health status dictionary
        """

        async def _check():
            # Health endpoints are at root, not under /api
            response = await self.http_client.get("/health")
            return response.json()

        return await retry_with_backoff(_check, self.retry_config)

    async def ready(self) -> dict[str, Any]:
        """Readiness check.

        Returns:
            Readiness status dictionary
        """

        async def _ready():
            # Health endpoints are at root, not under /api
            response = await self.http_client.get("/health/ready")
            return response.json()

        return await retry_with_backoff(_ready, self.retry_config)

    async def live(self) -> dict[str, Any]:
        """Liveness check.

        Returns:
            Liveness status dictionary
        """

        async def _live():
            # Health endpoints are at root, not under /api
            response = await self.http_client.get("/health/live")
            return response.json()

        return await retry_with_backoff(_live, self.retry_config)



"""Logging configuration for Vaulty SDK."""

import logging
import os
import sys


def get_logger(name: str) -> logging.Logger:
    """Get a logger instance for a module.

    Args:
        name: Logger name (typically __name__)

    Returns:
        Configured logger instance
    """
    logger = logging.getLogger(name)

    # Only configure if not already configured
    if not logger.handlers:
        logger.setLevel(_get_log_level())

        # Create console handler
        handler = logging.StreamHandler(sys.stderr)
        handler.setLevel(_get_log_level())

        # Create formatter
        formatter = logging.Formatter(
            fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
        )
        handler.setFormatter(formatter)

        logger.addHandler(handler)
        logger.propagate = False

    return logger


def _get_log_level() -> int:
    """Get log level from environment variable.

    Returns:
        Log level (default: WARNING)
    """
    level_str = os.getenv("VAULTY_LOG_LEVEL", "WARNING").upper()
    level_map = {
        "DEBUG": logging.DEBUG,
        "INFO": logging.INFO,
        "WARNING": logging.WARNING,
        "ERROR": logging.ERROR,
        "CRITICAL": logging.CRITICAL,
    }
    return level_map.get(level_str, logging.WARNING)


def sanitize_sensitive_data(data: dict) -> dict:
    """Sanitize sensitive data from logs.

    Args:
        data: Dictionary that may contain sensitive data

    Returns:
        Dictionary with sensitive fields redacted
    """
    sensitive_keys = {
        "password",
        "token",
        "api_token",
        "jwt_token",
        "access_token",
        "secret",
        "value",
        "authorization",
        "auth_header",
    }

    sanitized = {}
    for key, value in data.items():
        key_lower = key.lower()
        if any(sensitive in key_lower for sensitive in sensitive_keys):
            sanitized[key] = "***REDACTED***"
        elif isinstance(value, dict):
            sanitized[key] = sanitize_sensitive_data(value)
        else:
            sanitized[key] = value

    return sanitized
